
# @this file is responsible for running the downloader from
# - commandline.

from pygdl import download
from argpi import PathWays, Definition

definition = Definition()
definition.add('@in', 'file', '@f', 'The argument to specify a file')
definition.add('@git', 'git source', '@g', 'The git source in {username}/{repository} format.')
definition.add('@release', 'release version', '@r', "The github release version to look for. (Optional)")
definition.add('@out', 'outpath', '@o', 'The path where it should be downloaded (must exist)')


class cfg:
    file = ''
    git = ''
    release = 'latest'
    outpath = None
config = cfg()


arguments = PathWays(definition)
arguments.register('@in', lambda f: setattr(config, 'file', f), 'EXEC', what_value_expected='Single')
arguments.register('@git', lambda g: setattr(config, 'git', g), 'EXEC', what_value_expected='Single')
arguments.register('@release', lambda r: setattr(config, 'release', r), 'EXEC', what_value_expected='Single', ignore_if_not_present=True)
arguments.register('@out', lambda o: setattr(config, 'outpath', o), 'EXEC', what_value_expected='Single', ignore_if_not_present=True)
arguments.orchestrate


def main():
    try:
        download(config.file, config.git.split('/')[0], config.git.split('/')[1], config.release, config.outpath)
    except Exception:
        pass


if __name__.startswith('__main__'):
    status, e = download(config.file, config.git.split('/')[0], config.git.split('/')[1], config.release, config.outpath)
    if not status:
        raise RuntimeError(e)



# @this module contains tools for downloading files from
# - github release

from tqdm import tqdm

import typing
import pathlib
import requests


# @the github api for every release query.
GITHUB_API = "https://api.github.com/repos/{username}/{repository}/releases/{version}"


def download(
        filename: str,
        github_username: str,
        github_repository: str,
        release_version: str = 'latest',
        output_directory: str = None,
) -> typing.Tuple[bool, str]:
    # @customize the github_api based on given parameters
    api = GITHUB_API.replace('{username}', github_username) \
        .replace('{repository}', github_repository) \
        .replace('{version}', release_version)
    
    # @connect to the github api
    response = requests.get(api)

    # @try and check for any errors.
    try: response.raise_for_status()
    except: return False, "@network-failure"
    
    # @convert the response into json
    response_json = response.json()

    # @initialize a variable to None and
    # - try to find the filename given in the given release
    asset_url = None
    for asset in response_json['assets']:
        if asset['name'] == filename:
            asset_url = asset['browser_download_url']
            break
    
    # @if asset url is not found, return error
    if asset_url is None:
        return False, "@file-not-found"
    
    # @start the download process to the output directory.
    # - check if the output directory exists and is a directory.
    if output_directory is None:
        output_directory = pathlib.Path.cwd()
    outdir = pathlib.Path(output_directory).expanduser().resolve()

    if not outdir.exists() or not outdir.is_dir():
        return False, "@output-destination-must-be-a-directory"
    
    # - start the download
    response_d = requests.get(asset_url, stream=True)
    total_size = int(response_d.headers.get('content-length', 0))
    blocksize = 1024 # 1KB


    with open(outdir.joinpath(filename), 'wb') as f, tqdm(
        desc="Downloading " + filename,
        total=total_size,
        unit='iB',
        unit_scale=True,
        unit_divisor=blocksize,
        bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt}',
        ncols=90
    ) as tqdmbar:
        
        try:
            for data in response_d.iter_content(blocksize):
                f.write(data)
                tqdmbar.update(len(data))
        except KeyboardInterrupt:
            return False, "Interrupt"
    
    return True, ""